//+------------------------------------------------------------------+
//|                                                   fib levels.mq4 |
//|                                         Copyright  2011, brax64 |
//|                                         http://www.forex-tsd.com |
//+------------------------------------------------------------------+

// This indicator is using the core of the PivotsD_v5 indicator
// coded by traderathome (traderathome@msn.com) and Domas4.
// To them my gratitude and acknowledgment.

// This indicator calculate fibo levels in a different way from the classical calculation of
// daily pivots/support and resistence levels.
// It will calculate the range of the last closed bar (last 4 hours, last daily or the last weekly bar),
// then it sum/subtract, to the open of the new bar, the 23.6%, 38.2%, 61.8%, 82% of the calculated range.
// These levels can be used like supports or resistences and, mainly, to identify take profit
// and stop loss levels.
// i.e. when a level is broken there are chances that price will continue to the next one, or price can bounce off
// a level and will head to the next lower one
// Lines and labels are fully customizable, it have also the option to show/hide the daily/weekly pivot level.
// If Indicator_On = false, the levels are not shown, so you can temporary eliminate the levels from the chart
// without delete the indicator.
//
// brax64

#property copyright "Copyright  2011, brax64"
#property link      "http://www.forex-tsd.com"

#property indicator_chart_window

extern bool   Indicator_On                = true;
extern string note1                       = "Set time frame calculation with";
extern string note2                       = "        1=H4; 2=D1; 3=W1        ";
extern int    TF                          = 2;
extern string Part_1                      = "Level Lines Settings:";
extern bool   Show_D_W_Pivot              = true;
extern int    Pivot_LineStyle             = 0;
extern int    Pivot_LineThickness         = 2;
extern color  Pivot_LineColor             = DarkGray;
extern int    up_LineStyle                = 2;
extern int    up_LineThickness            = 0;
extern color  up_LineColor                = PowderBlue;
extern int    dn_LineStyle                = 2;
extern int    dn_LineThickness            = 0;
extern color  dn_LineColor                = Tomato;
extern string note3                       = "Start with fullscreen lines,   '1'";
extern string note4                       = "Start lines at Day Separator,  '2'";
extern string note5                       = "Start lines at current candle, '3'";
extern int    StartLines                  = 3;
extern bool   StopLines_At_Current_Candle = true;
extern string Part_2                      = "Level Labels Settings:";
extern color  up_LabelColor               = PowderBlue;
extern color  dn_LabelColor               = Tomato;
extern color  PivotLabels_Color           = Gray;
extern string Labels_FontStyle            = "Arial Black";
extern int    Labels_Fontsize             = 9; 
extern bool   Show_Price                  = true;
extern string note6                       = "Show_RightMargin_Prices and";
extern string note7                       = "Shift_Labels_PerCent_Left";
extern string note8                       = "work only when lines are fullscreen.";
extern bool   Show_RightMargin_Prices     = true;
extern int    Shift_Labels_PerCent_Left   = 100;
extern bool   Draw_Line_in_Background     = true;
extern bool   Draw_Labels_in_Background   = false;



int      fxDigits,shift,dow,CalcPeriod;
datetime StartTime;
double   HiPrice,LoPrice,Range,OpenPrice,ClosePrice;
double   U236,U382,U618,U820,D236,D382,D618,D820,Pivot;
string   Price;


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
  {
   string sub = StringSubstr(Symbol(), 3, 3);
      if (sub == "JPY")  fxDigits = 2;  else  fxDigits = 4;
//----
   return(0);
  }
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+

int deinit()
  {   
   int obj_total= ObjectsTotal();  
   for (int k= obj_total; k>=0; k--)
     {
      string name= ObjectName(k); 
      if (StringSubstr(name,0,10)=="Fib Levels") ObjectDelete(name);
     }    
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start()
{
   deinit();if (Indicator_On == false) return(0);

   string u236label, u382label, u618label, u820label;
   string d236label, d382label, d618label, d820label;
	string Plabel, tf;
	
   if (TF == 1) {CalcPeriod = PERIOD_H4; tf = "H4";}
   if (TF == 2) {CalcPeriod = PERIOD_D1; tf = "D";}
   if (TF == 3) {CalcPeriod = PERIOD_W1; tf = "W";}
 
         u236label = "+ 23.6% (" + tf + ")";
         u382label = "+ 38.2% (" + tf + ")";
         u618label = "+ 61.8% (" + tf + ")";
         u820label = "+ 82.0% (" + tf + ")";
         d236label = "- 23.6% (" + tf + ")";
         d382label = "- 38.2% (" + tf + ")";
         d618label = "- 61.8% (" + tf + ")";
         d820label = "- 82.0% (" + tf + ")";
         if (Show_D_W_Pivot && TF == 2)
         Plabel    = " D Pivot";
         if (Show_D_W_Pivot && TF == 3)
         Plabel    = " W Pivot";
          

   shift	= iBarShift(NULL,CalcPeriod,Time[0])+ 1;     //default = one period ago	 
	StartTime = iTime(NULL,CalcPeriod,shift);
	
	if (TF == 2)
	{
	dow = TimeDayOfWeek(StartTime);
	string dowStr;
	   switch(dow)
	      {
   	   case 5: dowStr = "Monday";break;
   	   //Monday = Friday value 2 periods ago 
         case 0: shift=iBarShift(NULL,CalcPeriod,Time[0])+2; dowStr = "Monday"; break;
         case 1: dowStr = "Tuesday"; break;       
         case 2: dowStr = "Wednesday"; break;
         case 3: dowStr = "Thursday"; break;
         case 4: dowStr = "Friday"; break;	 	   
         }
   }
         
   ClosePrice  = NormalizeDouble (iClose (NULL,CalcPeriod,shift),4);      
 	OpenPrice   = NormalizeDouble (iOpen  (NULL,CalcPeriod,0),4);
   HiPrice     = NormalizeDouble (iHigh  (NULL,CalcPeriod,shift),4); 
   LoPrice     = NormalizeDouble (iLow   (NULL,CalcPeriod,shift),4);
   Range       = HiPrice - LoPrice;
         
         U236  = OpenPrice + (Range*0.236);
         U382  = OpenPrice + (Range*0.382);
         U618  = OpenPrice + (Range*0.618);
         U820  = OpenPrice + (Range*0.820);
         D236  = OpenPrice - (Range*0.236);
         D382  = OpenPrice - (Range*0.382);
         D618  = OpenPrice - (Range*0.618);
         D820  = OpenPrice - (Range*0.820);
         if (Show_D_W_Pivot)
         Pivot = NormalizeDouble((HiPrice+LoPrice+ClosePrice)/3,4);

   if(up_LineStyle>0)    up_LineThickness    = 0;
   if(dn_LineStyle>0)    dn_LineThickness    = 0;
   if(Pivot_LineStyle>0) Pivot_LineThickness = 0;

      DrawLine  ("U236", U236, up_LineColor, up_LineStyle, up_LineThickness);
      DrawLabel (u236label, U236, up_LabelColor);
      DrawLine  ("U382", U382, up_LineColor, up_LineStyle, up_LineThickness);
      DrawLabel (u382label, U382, up_LabelColor);
      DrawLine  ("U618", U618, up_LineColor, up_LineStyle, up_LineThickness);
      DrawLabel (u618label, U618, up_LabelColor);
      DrawLine  ("U820", U820, up_LineColor, up_LineStyle, up_LineThickness);
      DrawLabel (u820label, U820, up_LabelColor);
      DrawLine  ("D236", D236, dn_LineColor, dn_LineStyle, dn_LineThickness);
      DrawLabel (d236label, D236, dn_LabelColor);
      DrawLine  ("D382", D382, dn_LineColor, dn_LineStyle, dn_LineThickness);
      DrawLabel (d382label, D382, dn_LabelColor);
      DrawLine  ("D618", D618, dn_LineColor, dn_LineStyle, dn_LineThickness);
      DrawLabel (d618label, D618, dn_LabelColor);
      DrawLine  ("D820", D820, dn_LineColor, dn_LineStyle, dn_LineThickness);
      DrawLabel (d820label, D820, dn_LabelColor);
   if (Show_D_W_Pivot && TF > 1)
   {
      DrawLine  ("PIVOT", Pivot,Pivot_LineColor,Pivot_LineStyle,Pivot_LineThickness);
      DrawLabel (Plabel, Pivot,PivotLabels_Color);
   }
   
   return(0);
}


//+-------------------------------------------------------------------------------------------+
//| drawLabel sub-routine to name and draw labels for level lines                             |                                                        
//+-------------------------------------------------------------------------------------------+ 

void DrawLabel(string text, double level, color Color)
   {  
   string plabel = "Fib Levels " + text + " Label";
   Price=DoubleToStr(level, fxDigits);
   if (Show_Price && StrToInteger(text)==0) text = text + "   " + Price;   
   
   if (Shift_Labels_PerCent_Left < 0)   {Shift_Labels_PerCent_Left =0;}
   if (Shift_Labels_PerCent_Left > 100) {Shift_Labels_PerCent_Left =100;}
   int L, LabelsShift = (WindowFirstVisibleBar()*Shift_Labels_PerCent_Left)/100;
   
   int time, startday = Time[1]; 
   if (Time[0] > iTime(NULL, CalcPeriod, 0)){startday = iTime(NULL, CalcPeriod, 0);} 
 
   string tab; 
   if (StartLines==1) L= 1;
   if (StartLines==2 && startday<=iTime(NULL,0,WindowFirstVisibleBar()) ) L= 1;
   if (L==1){
      time= Time[LabelsShift];
      if (Show_Price){tab="                                        ";}    //40                    
      else {tab = "                 ";}}                                  //17                    
   else { 
      if (Show_Price){tab="                                     ";}       //37
      else {tab = "              ";}                                      //14
      if (StartLines==2){time= startday;}
      else {time= Time[0];}}      

   if(ObjectFind(plabel) != 0){      
      ObjectCreate(plabel, OBJ_TEXT, 0, time, level);       
      ObjectSetText(plabel, tab+text, Labels_Fontsize, Labels_FontStyle, Color);
      ObjectSet(plabel, OBJPROP_BACK, false);
      if(Draw_Labels_in_Background) {ObjectSet(plabel, OBJPROP_BACK, true);}} 
   else{ObjectMove(plabel, 0, time, level);}            
   }
   
//+-------------------------------------------------------------------------------------------+
//| drawline sub-routine to name and draw levels lines                                        |                                                        
//+-------------------------------------------------------------------------------------------+ 
     
void DrawLine(string text, double level, color Color, int linestyle, int thickness)
   {
   string pline = "Fib Levels " + text + " Line";   
  
   int a = linestyle;
   int b = thickness;
   int c =1; if (a==0)c=b; 
   int F= false;                 //F= true= ray line
   int G= true;                  //G= true= subordinated lines
   int Z= OBJ_TREND; 
 
   int start, stop, startday = Time[1];
   if (Time[1] >= iTime(NULL,CalcPeriod,0)) startday = iTime(NULL,CalcPeriod,0);
    
   int L; if (StartLines==1) L= 1;
   if (StartLines==2 && startday<=iTime(NULL,0,WindowFirstVisibleBar())) L= 1;
   if (L==1) {{start= iTime(NULL,0,WindowFirstVisibleBar()); stop= Time[0]; F=true;}
      if (Show_RightMargin_Prices) {Z = OBJ_HLINE; 
         if (Draw_Line_in_Background) G= false;}}       
   else {if (StartLines==2) {start= startday; stop= Time[0]; Z= OBJ_TREND; F= true;}
   else {if (StartLines==3) {start= Time[1]; stop= Time[0]; Z= OBJ_TREND; F= true;} }}
   if (StopLines_At_Current_Candle && StartLines != 3) {Z= OBJ_TREND; F=false;}
       
   if(ObjectFind(pline) != 0){
      ObjectCreate(pline, Z, 0, start, level, stop, level);                            
      ObjectSet(pline, OBJPROP_STYLE, a);              
      ObjectSet(pline, OBJPROP_WIDTH, c);     
      ObjectSet(pline, OBJPROP_COLOR, Color);
      ObjectSet(pline, OBJPROP_BACK, G);
      ObjectSet(pline, OBJPROP_RAY, F);}      
      
   else{
      ObjectMove(pline, 0, start, level);         
      ObjectMove(pline, 1, stop, level);}  
   }

